@@ -24,4 +24,4 @@ def main(args):
         ELF(e.path)
 
 if __name__ == '__main__':
-    pwnlib.commandline.common.main(__file__)
+    pwnlib.commandline.common.main(__file__, main)
